/*This class will handle all of the images and thumbnails*/

#include "ImageManager.h"

// Constructor and Destructor
ImageManager::ImageManager(){}
ImageManager::~ImageManager()
{
  blepo::ImgBgr *img_ptr;

  //Unallocate memory from vector
  for(int i = m_image_vector.size(); i > 1; i--)
  {
    img_ptr = m_image_vector.back();
    delete img_ptr;
    m_image_vector.pop_back();
  }
}

// Sets up map data structures
void ImageManager::SetDirectoryAndImageFileFormat(const CString& dir, const ImageFileFormat& iff)
{
  // Set up variables
  CFileFind ff;
  BOOL b_find;
  ImageInfo image_info;
  blepo::ImgBgr *image_ptr;
  CString file_name;
  CString path = dir + "\\";
  
  
  // Set up cfilefind
  b_find = ff.FindFile(path+"*.jpg");

  while(b_find)
  {
    // Add blank element to vector
    AllocateMemory();
    image_ptr = m_image_vector.back();

    // Find next image file
    b_find = ff.FindNextFile();

    // Grab file info
    file_name = ff.GetFileName();
    image_info = iff.GetImageInfo(file_name);
    
    blepo::Load(path+file_name, image_ptr);
    
    // store image in map
    StoreImage(image_info.tube, image_info.window, image_info.session, image_ptr, m_image_map);
  } 
}

// Stores common images and managers cache size
void ImageManager::SetCurrentTws(const CString& tube, int window, int session)
{
  // Cache image
  blepo::ImgBgr *image_ptr = NULL;
  GetImage(tube, window, session, image_ptr);
  
  if(m_image_cache.size() < CACHE_SIZE)
    StoreImage(tube, window, session, image_ptr, m_image_cache);  //there is room in the cache

  else
  {
    m_image_cache.erase( m_image_cache.begin() );   // make room in cache
    StoreImage(tube, window, session, image_ptr, m_image_cache);
  }
}

// Gets image from map
void ImageManager::GetImage(const CString& tube, int window, int session, blepo::ImgBgr *out) const
{
  //Sets out equal to an ImgBgr given a tube, window, and session, or NULL if unsuccessful
  std::string key;
  key += tube;
  key += window;
  key += session;

  // Search through cache map using t/w/s
  ImageMap::const_iterator it = m_image_cache.find(key);

  if(it != m_image_map.end())
  {  
    out = it->second;
    return;
  }
  else
    out = NULL; // image not found


  // Search through image map using t/w/s
  if(it != m_image_map.end())     out = it->second;   // it->second is the value stored in hash (first is the key)
  else                            out = NULL;         // image not found
}

//Same functionality as GetImage for now, in future return smaller image
void ImageManager::GetThumbnail(const CString& tube, int window, int session, blepo::ImgBgr* out) const
{
  //Sets out equal to an ImgBgr given a tube, window, and session, or -1 if unsuccessful
  std::string key;
  key += tube;
  key += window;
  key += session;

  // Search through cache map using t/w/s
  ImageMap::const_iterator it = m_image_cache.find(key);

  if(it != m_image_map.end())
  {  
    out = it->second;
    return;
  }
  else
    out = NULL; // image not found

  // Search through image map using t/w/s
  if(it != m_image_map.end())     out = it->second;
  else                            out = NULL;         // image not found
}

// Builds key and stores the image pointer in the proper map
void ImageManager::StoreImage(const CString& tube, int window, int session, blepo::ImgBgr *image_ptr, ImageMap &image_map)
{
  std::string key;
  key += tube;
  key += window;
  key += session;
  image_map.insert(key_pair (key, image_ptr));
}

// Allocates memory for image pointer
void ImageManager::AllocateMemory()
{
  m_blank_image = new blepo::ImgBgr;
  m_image_vector.push_back(m_blank_image);
}