#if !defined(AFX_PROGRESSDLG_H__3DFEA1A8_A703_47AF_A60B_4DA9D36BCD43__INCLUDED_)
#define AFX_PROGRESSDLG_H__3DFEA1A8_A703_47AF_A60B_4DA9D36BCD43__INCLUDED_

#include "DataStructures.h"

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// ProgressDlg.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// ProgressDlg dialog
// This class handles the Progress Dialog, which keeps track of which images
// in the repository have been annotated.
// Un-annotated images will show up here as red, annotated ones will be green,
// and non-existent or invalid images will be grey.
// 
// The dialog shows a grid of squares representing the images, which are 
// scrollable and (eventually) clickable, which will close the progress 
// window and bring up that clicked image in the main rootfly window.
//
// The bar at the bottom of the progress window allows you to easily switch
// between tubes, by clicking on the desired one. Also, the color shows you
// if the entire tube of images has been annotated (green) or if some or none
// have been annotated (red). 
/////////////////////////////////////////////////////////////////////////////



class ProgressDlg; // forward declaration

class ProgressDlg : public CDialog
{
// Construction
public:
	void InitSelection();
  ProgressDlg::ProgressDlg(const Experiment &experiment, const CString &tube, CWnd* pParent = NULL);
  void SelectImage(const int &tube, const int &session, const int &window);
  
  //These are public because RootflyDlg calls them from outside this class
  int m_selected_tube;
  int m_selected_session;
  int m_selected_window;

private:
  const std::vector<Tube>* m_tube_list;
  int m_max_session;
  int m_max_window;
  CString m_displayed_tube;
  int m_starting_session;
  int m_starting_window;
  int m_jump_size;
  int m_columns;
  int m_rows;
  int m_session;
  CToolTipCtrl m_tool_tip;
	CSize m_control_size;
  CSize m_tube_size;
  CSize m_window_size;
	int m_control_margin;
	int m_tube_spacing;
	int m_tube_margin;
  int m_tube_top;
	int m_window_spacing;
  int m_row_length;
  int m_column_length;
  int m_label_height;
  bool m_ChangeCursor;

	void DrawWindow(const CRect &window, const COLORREF &color, CPaintDC *dc);
	void DrawLegend(CPaintDC *dc);
  void ReDraw();
  void DrawTubeSelect(CPaintDC *dc);
  CString StripPaddedZeros(CString input);

  COLORREF COLOR_NOT_EXIST;
  COLORREF COLOR_NOT_ANNOTATED;
  COLORREF COLOR_DONE;

// Dialog Data
	//{{AFX_DATA(ProgressDlg)
	enum { IDD = IDD_PROGRESS_DIALOG };
	CButton	m_down_button;
	CButton	m_left_button;
	CButton	m_right_button;
	CButton	m_up_button;
	CScrollBar	m_sTubeSelect;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(ProgressDlg)
	public:
	virtual void OnFinalRelease();
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(ProgressDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnPaint();
	afx_msg void OnClickUp();
	afx_msg void OnClickRight();
	afx_msg void OnClickLeft();
	afx_msg void OnClickDown();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_PROGRESSDLG_H__3DFEA1A8_A703_47AF_A60B_4DA9D36BCD43__INCLUDED_)
